import h5py
import os
import numpy as np
import pandas as pd

full_post_fname = '_iid_mag_two_comp_iid_tilt_powerlaw_redshift_full_posterior_draw_45_events_'
map_mass_model_to_names = {'peak' : 'mass_c', 'BPL' : 'mass_d'}
map_BBHsel_to_names = {
    'NoAugNoEvolution': 'excludes_190814',
}

def read_samples(filename):
    with h5py.File(filename, "r") as ff:
        data = ff["samples"][:]
        events = [event.decode() for event in ff["events"]]
    data_frame = pd.DataFrame()
    for name in data.dtype.names:
        data_frame[name] = data[name]
    return data_frame, events


def path_to_file(BBHsel_name, mass_model):
    mass = map_mass_model_to_names[mass_model]
    BBHsel = map_BBHsel_to_names[BBHsel_name]
    return 'o1o2o3_' + mass + full_post_fname + BBHsel + '.hdf5'


def read_observed_data(fname):
    synthetic_data = []
    data, events = read_samples(fname)
    # ignoring number of events using fixed number instead                                                                                                                                                  
    n_events = 45
    parameters = [
        "_".join(key.split("_")[1:]) for key in data
        if events[0] in key and "prior" not in key
    ]

    synthetic = {
        key: np.array([
            data[f"synthetic_{key}_{ii}"] for ii in range(n_events)
            if f"synthetic_{key}_{ii}" in data
        ])
        for key in parameters
        if any([f"synthetic_{key}_{ii}" in data for ii in range(n_events)])
    }

    synthetic["mass_2"] = synthetic["mass_1"]*synthetic["mass_ratio"]
    return synthetic


source_min_m2 = 2.6
source_min_q = 0.11353

stat_file =  open("GW190814_outlier_stats.txt","w")
stat_dict = {}

for BBHsel_name in ['NoAugNoEvolution']:
    for mass_model in map_mass_model_to_names.keys():
        if os.path.exists(path_to_file(BBHsel_name, mass_model)):

            synthetic = read_observed_data(path_to_file(BBHsel_name, mass_model))

            all_m2_samples = []
            all_q_samples = []
            for j in range(5000):
                m2_samples = []
                q_samples = []
                values1 = synthetic["mass_2"]
                values2 = synthetic["mass_ratio"]
                for i in range(45):
                    samples1 = values1[i]
                    samples2 = values2[i]
                    sample1 = samples1[j]
                    sample2 = samples2[j]
                    m2_samples.append(sample1)
                    q_samples.append(sample2)
                all_m2_samples.append(np.min(m2_samples))
                all_q_samples.append(np.min(q_samples))

            all_m2_samples = np.array(all_m2_samples)
            min_m2 = min(all_m2_samples)

            if min_m2 <= source_min_m2:
                m2_below_thres = all_m2_samples[all_m2_samples<source_min_m2]
                percent_m2 = len(m2_below_thres)*100/len(all_m2_samples)
                per_m2 = "<= {}".format(percent_m2)
            else:
                percent_m2 = 100./len(all_m2_samples)
                per_m2 = "< {}".format(percent_m2)
                
            all_q_samples = np.array(all_q_samples)
            min_q = min(all_q_samples)

            if min_q <= source_min_q:
                q_below_thres = all_q_samples[all_q_samples < source_min_q]
                percent_q = len(q_below_thres)*100/len(all_q_samples)
                per_q = "<= {}".format(percent_q)
            else:
                percent_q = 100.0/len(all_q_samples)
                per_q = "< {}".format(percent_q)


            stat_dict[mass_model+BBHsel_name+"MTwoMinPercentBelow"] = [
                'percentage of samples below m2 for GW190814 for'+path_to_file(BBHsel_name, mass_model),
                percent_m2,
            ]

            stat_dict[mass_model+BBHsel_name+"MassRatioMinPercentBelow"] = [
                'percentage of samples below q for GW190814 for'+path_to_file(BBHsel_name, mass_model),
                percent_q,
            ]

            print(f"samples below m2 of GW190814 {per_m2} %")
            print(f"minimum m2 = {min_m2}")
            print(f"samples below q of GW190814 {per_q} %")
            print(f"minimum q  = {min_q}")

        else:
             print(path_to_file(BBHsel_name, mass_model), ' does not exist')

for key in stat_dict:
    stat_file.write("%"+stat_dict[key][0]+"\n")
    stat_file.write("\\newcommand{\\%s}{\ensuremath{%s}}\n" % (key, round(stat_dict[key][1],2)))
    stat_file.write("\n")
stat_file.close()
